@@ -2,7 +2,6 @@
 //! Use this for any vector that is serialised.
 
 use std::{
-    convert::TryInto,
     io::{Read, Write},
     ops::Drop,
 };
@@ -11,10 +10,238 @@ use std::{
 use serde::ser::SerializeStruct;
 use zeroize::Zeroize;
 
-use crate::{Deserialize, Error, Serialize, TlsSize};
+use crate::{Deserialize, Error, Serialize, Size};
+
+macro_rules! impl_size {
+    ($self:ident, $size:ty, $name:ident, $len_len:literal) => {
+        /// The serialized len
+        #[inline(always)]
+        fn tls_serialized_length(&$self) -> usize {
+            $self.as_slice()
+                .iter()
+                .fold($len_len, |acc, e| acc + e.tls_serialized_len())
+        }
+    }
+}
+
+macro_rules! impl_byte_size {
+    ($self:ident, $size:ty, $name:ident, $len_len:literal) => {
+        /// The serialized len
+        #[inline(always)]
+        fn tls_serialized_byte_length(&$self) -> usize {
+            $self.as_slice().len() + $len_len
+        }
+    }
+}
+
+macro_rules! impl_byte_deserialize {
+    ($self:ident, $size:ty, $name:ident, $len_len:literal) => {
+        #[inline(always)]
+        fn deserialize_bytes<R: Read>(bytes: &mut R) -> Result<Self, Error> {
+            let len = <$size>::tls_deserialize(bytes)? as usize;
+            let mut result = Self {
+                vec: Vec::with_capacity(len),
+            };
+            let read = bytes.read(result.vec.as_mut_slice())?;
+            debug_assert_eq!(read, len);
+            if read != len {
+                Err(Error::DecodingError(format!(
+                    "{} bytes were read but {} were expected",
+                    read, len
+                )))
+            } else {
+                Ok(result)
+            }
+        }
+    };
+}
+
+macro_rules! impl_deserialize {
+    ($self:ident, $size:ty, $name:ident, $len_len:literal) => {
+        #[inline(always)]
+        fn deserialize<R: Read>(bytes: &mut R) -> Result<Self, Error> {
+            let mut result = Self { vec: Vec::new() };
+            let len = <$size>::tls_deserialize(bytes)?;
+            let mut read = len.tls_serialized_len();
+            let len_len = read;
+            while (read - len_len) < len as usize {
+                let element = T::tls_deserialize(bytes)?;
+                read += element.tls_serialized_len();
+                result.push(element);
+            }
+            Ok(result)
+        }
+    };
+}
+
+macro_rules! impl_serialize {
+    ($self:ident, $size:ty, $name:ident, $len_len:literal) => {
+        #[inline(always)]
+        fn serialize<W: Write>(&$self, writer: &mut W) -> Result<usize, Error> {
+            // Get the byte length of the content, make sure it's not too
+            // large and write it out.
+            let tls_serialized_len = $self.tls_serialized_len();
+            let byte_length = tls_serialized_len - $len_len;
+
+            let max_len = <$size>::MAX as usize;
+            debug_assert!(
+                byte_length <= max_len,
+                "Vector length can't be encoded in the vector length a {} >= {}",
+                byte_length,
+                max_len
+            );
+            if byte_length > max_len {
+                return Err(Error::InvalidVectorLength);
+            }
+
+            let mut written = (byte_length as $size).tls_serialize(writer)?;
+
+            // Now serialize the elements
+            for e in $self.as_slice().iter() {
+                written += e.tls_serialize(writer)?;
+            }
+
+            debug_assert_eq!(
+                written, tls_serialized_len,
+                "{} bytes should have been serialized but {} were written",
+                tls_serialized_len, written
+            );
+            if written != tls_serialized_len {
+                return Err(Error::EncodingError(format!(
+                    "{} bytes should have been serialized but {} were written",
+                    tls_serialized_len, written
+                )));
+            }
+            Ok(written)
+        }
+    };
+}
+
+macro_rules! impl_byte_serialize {
+    ($self:ident, $size:ty, $name:ident, $len_len:literal) => {
+        #[inline(always)]
+        fn serialize_bytes<W: Write>(&$self, writer: &mut W) -> Result<usize, Error> {
+            // Get the byte length of the content, make sure it's not too
+            // large and write it out.
+            let tls_serialized_len = $self.tls_serialized_len();
+            let byte_length = tls_serialized_len - $len_len;
+
+            let max_len = <$size>::MAX as usize;
+            debug_assert!(
+                byte_length <= max_len,
+                "Vector length can't be encoded in the vector length a {} >= {}",
+                byte_length,
+                max_len
+            );
+            if byte_length > max_len {
+                return Err(Error::InvalidVectorLength);
+            }
+
+            let mut written = (byte_length as $size).tls_serialize(writer)?;
+
+            // Now serialize the elements
+            written += writer.write($self.as_slice())?;
+
+            debug_assert_eq!(
+                written, tls_serialized_len,
+                "{} bytes should have been serialized but {} were written",
+                tls_serialized_len, written
+            );
+            if written != tls_serialized_len {
+                return Err(Error::EncodingError(format!(
+                    "{} bytes should have been serialized but {} were written",
+                    tls_serialized_len, written
+                )));
+            }
+            Ok(written)
+        }
+    };
+}
+
+macro_rules! impl_tls_vec_codec_generic {
+    ($size:ty, $name:ident, $len_len: literal, $($bounds:ident),*) => {
+        impl<T: $($bounds + )*> Serialize
+            for $name<T>
+        {
+            fn tls_serialize<W: Write>(&self, writer: &mut W) -> Result<usize, Error> {
+                self.serialize(writer)
+            }
+        }
+
+        impl<T: $($bounds + )*> Size
+            for $name<T>
+        {
+            #[inline]
+            fn tls_serialized_len(&self) -> usize {
+                self.tls_serialized_length()
+            }
+        }
+
+        impl<T: $($bounds + )*> Serialize
+            for &$name<T>
+        {
+            fn tls_serialize<W: Write>(&self, writer: &mut W) -> Result<usize, Error> {
+                self.serialize(writer)
+            }
+        }
+
+        impl<T: $($bounds + )*> Size
+            for &$name<T>
+        {
+            #[inline]
+            fn tls_serialized_len(&self) -> usize {
+                self.tls_serialized_length()
+            }
+        }
+
+        impl<T: $($bounds + )*> Deserialize
+            for $name<T>
+        {
+            fn tls_deserialize<R: Read>(bytes: &mut R) -> Result<Self, Error> {
+                Self::deserialize(bytes)
+            }
+        }
+    }
+}
+
+macro_rules! impl_tls_vec_codec_bytes {
+    ($size:ty, $name:ident, $len_len: literal) => {
+        impl Serialize for $name<u8> {
+            fn tls_serialize<W: Write>(&self, writer: &mut W) -> Result<usize, Error> {
+                self.serialize_bytes(writer)
+            }
+        }
+
+        impl Size for $name<u8> {
+            #[inline]
+            fn tls_serialized_len(&self) -> usize {
+                self.tls_serialized_byte_length()
+            }
+        }
+
+        impl Serialize for &$name<u8> {
+            fn tls_serialize<W: Write>(&self, writer: &mut W) -> Result<usize, Error> {
+                self.serialize_bytes(writer)
+            }
+        }
+
+        impl Size for &$name<u8> {
+            #[inline]
+            fn tls_serialized_len(&self) -> usize {
+                self.tls_serialized_byte_length()
+            }
+        }
+
+        impl Deserialize for $name<u8> {
+            fn tls_deserialize<R: Read>(bytes: &mut R) -> Result<Self, Error> {
+                Self::deserialize_bytes(bytes)
+            }
+        }
+    };
+}
 
 macro_rules! impl_tls_vec {
-    ($size:ty, $name:ident, $len_len: literal, $($bounds: ident),*) => {
+    ($size:ty, $name:ident, $len_len: literal, $($bounds:ident),*) => {
         #[derive(Eq, Clone, Debug)]
         pub struct $name<T: $($bounds + )*> {
             vec: Vec<T>,
@@ -104,69 +331,9 @@ macro_rules! impl_tls_vec {
             pub fn len_len() -> usize {
                 $len_len
             }
-
-            /// The serialization
-            #[inline(always)]
-            fn serialize<W: Write>(&self, writer: &mut W) -> Result<usize, Error> {
-                let len = self.vec.len();
-                let serialized_len = self.serialized_len();
-                let mut temp_buf = Vec::<u8>::with_capacity(serialized_len - $len_len);
-                let max_len = <$size>::MAX as usize;
-                if len > max_len {
-                    return Err(Error::InvalidVectorLength);
-                }
-                let mut written = 0usize;
-                for e in self.vec.iter() {
-                    written += e.tls_serialize(&mut temp_buf)?;
-                }
-                let max_len = max_len - $len_len;
-                debug_assert!(written < max_len);
-                if written > max_len {
-                    return Err(Error::InvalidVectorLength);
-                }
-                debug_assert_eq!(written, serialized_len - $len_len);
-                if written != serialized_len - $len_len {
-                    return Err(Error::EncodingError(format!("{} bytes should have been serialized but {} were written", serialized_len, written)));
-                }
-                written += (written as $size).tls_serialize(writer)?;
-                writer.write_all(&temp_buf)?;
-                Ok(written)
-            }
-
-            /// The deserialization
-            #[inline(always)]
-            fn deserialize<R: Read>(bytes: &mut R) -> Result<Self, Error> {
-                let mut result = Self { vec: Vec::new() };
-                let len = <$size>::tls_deserialize(bytes)?;
-                let mut sub = bytes.take(len.try_into()?);
-                loop {
-                    // XXX: check if this is efficient enough for things like u8
-                    let element = T::tls_deserialize(&mut sub);
-                    let element = match element {
-                        Ok(e) => e,
-                        Err(e) => {
-                            if e == Error::EndOfStream {
-                                break;
-                            } else {
-                                return Err(e);
-                            }
-                        }
-                    };
-                    result.push(element);
-                }
-                Ok(result)
-            }
-
-            /// The serialized len
-            #[inline(always)]
-            fn serialized_length(&self) -> usize {
-                self.vec
-                    .iter()
-                    .fold($len_len, |acc, e| acc + e.serialized_len())
-            }
         }
 
-        impl<T: std::hash::Hash+ $($bounds + )*> std::hash::Hash for $name<T> {
+        impl<T: std::hash::Hash + $($bounds + )*> std::hash::Hash for $name<T> {
             #[inline]
             fn hash<H: std::hash::Hasher>(&self, state: &mut H) {
                 self.vec.hash(state)
@@ -270,7 +437,7 @@ macro_rules! impl_tls_vec {
                 + Deserialize
                 + Clone
                 + PartialEq
-                + TlsSize
+                + Size
                 + $($bounds + )*
                 serde::de::Deserialize<'de>,
         {
@@ -324,7 +491,7 @@ macro_rules! impl_tls_vec {
                         + Deserialize
                         + Clone
                         + PartialEq
-                        + TlsSize
+                        + Size
                         + $($bounds + )*
                         serde::de::Deserialize<'de>,
                 {
@@ -369,48 +536,6 @@ macro_rules! impl_tls_vec {
                 )
             }
         }
-
-        impl<T: $($bounds + )*> Serialize
-            for $name<T>
-        {
-            fn tls_serialize<W: Write>(&self, writer: &mut W) -> Result<usize, Error> {
-                self.serialize(writer)
-            }
-        }
-
-        impl<T: $($bounds + )*> TlsSize
-            for $name<T>
-        {
-            #[inline]
-            fn serialized_len(&self) -> usize {
-                self.serialized_length()
-            }
-        }
-
-        impl<T: $($bounds + )*> Serialize
-            for &$name<T>
-        {
-            fn tls_serialize<W: Write>(&self, writer: &mut W) -> Result<usize, Error> {
-                self.serialize(writer)
-            }
-        }
-
-        impl<T: $($bounds + )*> TlsSize
-            for &$name<T>
-        {
-            #[inline]
-            fn serialized_len(&self) -> usize {
-                self.serialized_length()
-            }
-        }
-
-        impl<T: $($bounds + )*> Deserialize
-            for $name<T>
-        {
-            fn tls_deserialize<R: Read>(bytes: &mut R) -> Result<Self, Error> {
-                Self::deserialize(bytes)
-            }
-        }
     };
 }
 
@@ -424,19 +549,35 @@ macro_rules! impl_secret_tls_vec {
             Deserialize,
             Clone,
             PartialEq,
-            TlsSize,
+            Size,
+            Zeroize
+        );
+        impl_tls_vec_codec_generic!(
+            $size,
+            $name,
+            $len_len,
+            Serialize,
+            Deserialize,
+            Clone,
+            PartialEq,
+            Size,
             Zeroize
         );
 
-        impl<T: Serialize + Deserialize + Clone + PartialEq + TlsSize + Zeroize> Zeroize
-            for $name<T>
-        {
+        impl<T: Serialize + Deserialize + Clone + PartialEq + Size + Zeroize> $name<T> {
+            // This implements serialize and size for all versions
+            impl_serialize!(self, $size, $name, $len_len);
+            impl_size!(self, $size, $name, $len_len);
+            impl_deserialize!(self, $size, $name, $len_len);
+        }
+
+        impl<T: Serialize + Deserialize + Clone + PartialEq + Size + Zeroize> Zeroize for $name<T> {
             fn zeroize(&mut self) {
                 self.vec.zeroize()
             }
         }
 
-        impl<T: Serialize + Deserialize + Clone + PartialEq + TlsSize + Zeroize> Drop for $name<T> {
+        impl<T: Serialize + Deserialize + Clone + PartialEq + Size + Zeroize> Drop for $name<T> {
             fn drop(&mut self) {
                 self.zeroize()
             }
@@ -454,15 +595,63 @@ macro_rules! impl_public_tls_vec {
             Deserialize,
             Clone,
             PartialEq,
-            TlsSize
+            Size
         );
+
+        impl_tls_vec_codec_generic!(
+            $size,
+            $name,
+            $len_len,
+            Serialize,
+            Deserialize,
+            Clone,
+            PartialEq,
+            Size
+        );
+
+        impl<T: Serialize + Deserialize + Clone + PartialEq + Size> $name<T> {
+            // This implements serialize and size for all versions
+            impl_serialize!(self, $size, $name, $len_len);
+            impl_size!(self, $size, $name, $len_len);
+            impl_deserialize!(self, $size, $name, $len_len);
+        }
     };
 }
 
+macro_rules! impl_public_tls_byte_vec {
+    ($size:ty, $name:ident, $len_len: literal) => {
+        impl_tls_vec!(
+            $size,
+            $name,
+            $len_len,
+            Serialize,
+            Deserialize,
+            Clone,
+            PartialEq,
+            Size
+        );
+
+        impl $name<u8> {
+            // This implements serialize and size for all versions
+            impl_byte_serialize!(self, $size, $name, $len_len);
+            impl_byte_size!(self, $size, $name, $len_len);
+            impl_byte_deserialize!(self, $size, $name, $len_len);
+        }
+
+        impl_tls_vec_codec_bytes!($size, $name, $len_len);
+    };
+}
+
+// TODO: #1 provide specialized TlsByteVec* versions holding u8 that are more efficient.
+
 impl_public_tls_vec!(u8, TlsVecU8, 1);
 impl_public_tls_vec!(u16, TlsVecU16, 2);
 impl_public_tls_vec!(u32, TlsVecU32, 4);
 
+impl_public_tls_byte_vec!(u8, TlsByteVecU8, 1);
+impl_public_tls_byte_vec!(u16, TlsByteVecU16, 2);
+impl_public_tls_byte_vec!(u32, TlsByteVecU32, 4);
+
 // Secrets should be put into these Secret tls vectors as they implement zeroize.
 impl_secret_tls_vec!(u8, SecretTlsVecU8, 1);
 impl_secret_tls_vec!(u16, SecretTlsVecU16, 2);
@@ -470,74 +659,93 @@ impl_secret_tls_vec!(u32, SecretTlsVecU32, 4);
 
 // We also implement shallow serialization for slices
 
+macro_rules! impl_tls_byte_slice {
+    ($size:ty, $name:ident, $len_len:literal) => {
+        pub struct $name<'a>(pub &'a [u8]);
+
+        impl<'a> $name<'a> {
+            #[inline(always)]
+            fn as_slice(&self) -> &[u8] {
+                &self.0
+            }
+        }
+
+        impl<'a> $name<'a> {
+            impl_byte_serialize!(self, $size, $name, $len_len);
+            impl_byte_size!(self, $size, $name, $len_len);
+        }
+
+        impl<'a> Serialize for &$name<'a> {
+            fn tls_serialize<W: Write>(&self, writer: &mut W) -> Result<usize, Error> {
+                self.serialize_bytes(writer)
+            }
+        }
+
+        impl<'a> Serialize for $name<'a> {
+            fn tls_serialize<W: Write>(&self, writer: &mut W) -> Result<usize, Error> {
+                self.serialize_bytes(writer)
+            }
+        }
+
+        impl<'a> Size for &$name<'a> {
+            #[inline]
+            fn tls_serialized_len(&self) -> usize {
+                self.tls_serialized_byte_length()
+            }
+        }
+
+        impl<'a> Size for $name<'a> {
+            #[inline]
+            fn tls_serialized_len(&self) -> usize {
+                self.tls_serialized_byte_length()
+            }
+        }
+    };
+}
+
+impl_tls_byte_slice!(u8, TlsByteSliceU8, 1);
+impl_tls_byte_slice!(u16, TlsByteSliceU16, 2);
+impl_tls_byte_slice!(u32, TlsByteSliceU32, 4);
+
 macro_rules! impl_tls_slice {
     ($size:ty, $name:ident, $len_len: literal) => {
-        pub struct $name<'a, T: TlsSize + Serialize>(pub &'a [T]);
+        pub struct $name<'a, T: Size + Serialize>(pub &'a [T]);
 
-        impl<'a, T: TlsSize + Serialize> $name<'a, T> {
-            fn serialize<W: Write>(&self, writer: &mut W) -> Result<usize, Error> {
-                let len = self.0.len();
-                let serialized_len = self.serialized_len();
-                let max_len = <$size>::MAX as usize;
-                if len > max_len {
-                    return Err(Error::InvalidVectorLength);
-                }
-
-                // Write the content of the slice to a temp buffer.
-                let mut written = 0usize;
-                let serialized_payload_len = serialized_len - $len_len;
-                let mut temp_buf = Vec::<u8>::with_capacity(serialized_payload_len);
-                for e in self.0.iter() {
-                    written += e.tls_serialize(&mut temp_buf)?;
-                }
-                let max_len = max_len - 1;
-                debug_assert!(written < max_len);
-                if written > max_len {
-                    return Err(Error::InvalidVectorLength);
-                }
-                debug_assert_eq!(written, serialized_payload_len);
-                if written != serialized_payload_len {
-                    return Err(Error::EncodingError(format!(
-                        "{} bytes should have been serialized but {} were written",
-                        serialized_len, written
-                    )));
-                }
-
-                // Actually write to the output writer
-                written += (written as $size).tls_serialize(writer)?;
-                writer.write_all(&temp_buf)?;
-
-                Ok(written)
+        impl<'a, T: Size + Serialize> $name<'a, T> {
+            #[inline(always)]
+            fn as_slice(&self) -> &[T] {
+                &self.0
             }
         }
 
-        impl<'a, T: TlsSize + Serialize> Serialize for &$name<'a, T> {
+        impl<'a, T: Size + Serialize> $name<'a, T> {
+            impl_serialize!(self, $size, $name, $len_len);
+            impl_size!(self, $size, $name, $len_len);
+        }
+
+        impl<'a, T: Size + Serialize> Serialize for &$name<'a, T> {
             fn tls_serialize<W: Write>(&self, writer: &mut W) -> Result<usize, Error> {
                 self.serialize(writer)
             }
         }
 
-        impl<'a, T: TlsSize + Serialize> Serialize for $name<'a, T> {
+        impl<'a, T: Size + Serialize> Serialize for $name<'a, T> {
             fn tls_serialize<W: Write>(&self, writer: &mut W) -> Result<usize, Error> {
                 self.serialize(writer)
             }
         }
 
-        impl<'a, T: TlsSize + Serialize> TlsSize for &$name<'a, T> {
+        impl<'a, T: Size + Serialize> Size for &$name<'a, T> {
             #[inline]
-            fn serialized_len(&self) -> usize {
-                self.0
-                    .iter()
-                    .fold($len_len, |acc, e| acc + e.serialized_len())
+            fn tls_serialized_len(&self) -> usize {
+                self.tls_serialized_length()
             }
         }
 
-        impl<'a, T: TlsSize + Serialize> TlsSize for $name<'a, T> {
+        impl<'a, T: Size + Serialize> Size for $name<'a, T> {
             #[inline]
-            fn serialized_len(&self) -> usize {
-                self.0
-                    .iter()
-                    .fold($len_len, |acc, e| acc + e.serialized_len())
+            fn tls_serialized_len(&self) -> usize {
+                self.tls_serialized_length()
             }
         }
     };
